import argparse

from .. lib import (
    eb,
    parameters
)

import boto


def get_argument_parser():
    parser = argparse.ArgumentParser("ebzl env")
    
    parameters.add_profile(parser)
    parameters.add_app_name(parser)
    parameters.add_region(parser, required=False)

    parser.add_argument("-e", "--env-name",
                        required=True,
                        help="ElasticBeanstalk environment name.")

    return parser


def env(args):
    layer1 = eb.get_layer1(profile=args.profile, region=args.region)

    try:
        data = layer1.describe_configuration_settings(
            application_name=args.app_name,
            environment_name=args.env_name)
    except boto.exception.BotoServerError as exc:
        print(exc.message)
        return

    env_vars = (data["DescribeConfigurationSettingsResponse"]
                    ["DescribeConfigurationSettingsResult"]
                    ["ConfigurationSettings"]
                    [0]
                    ["OptionSettings"])

    aws_env_var_option = "aws:elasticbeanstalk:application:environment"

    env_vars = {v["OptionName"]: v["Value"] for v in env_vars
                if v["Namespace"] == aws_env_var_option}

    for key, value in env_vars.items():
        print("%s = %s" % (key, value))


def run(argv):
    args = parameters.parse(parser=get_argument_parser(),
                            argv=argv,
                            postprocessors=[parameters.add_default_region])
    
    env(args)



import boto

from . import config


def get_connection(profile_name):
    return boto.connect_s3(*config.get_credentials(profile_name))


def map_region_to_location(region):
    if region.startswith("eu-"):
        return "EU"
    return region



import argparse

from .. lib import (
    parameters,
    eb,
    ec2
)

import boto


def get_argument_parser():
    parser = argparse.ArgumentParser("ebzl instances")
    
    parameters.add_profile(parser)
    parameters.add_region(parser, required=False)

    group = parser.add_mutually_exclusive_group(required=True)

    eb_group = group.add_argument_group()
    eb_group.add_argument(*parameters.APP_ARGS, **parameters.APP_KWARGS)
    eb_group.add_argument("-e", "--env-name",
                          help="ElasticBeanstalk environment name.")

    group.add_argument("-n", "--name",
                       help="Instance Name (tag).")

    return parser


def env(args):
    layer1 = eb.get_layer1(profile=args.profile, region=args.region)

    try:
        data = layer1.describe_environment_resources(
            environment_name=args.env_name)
    except boto.exception.BotoServerError as exc:
        print(exc.message)
        return

    instance_ids = (data["DescribeEnvironmentResourcesResponse"]
                        ["DescribeEnvironmentResourcesResult"]
                        ["EnvironmentResources"]
                        ["Instances"])

    instance_ids = [x["Id"] for x in instance_ids]

    ec2_conn = ec2.get_connection(args.profile, args.region)
    instances = ec2_conn.get_only_instances(instance_ids=instance_ids)

    for i in instances:
        print("%s-%s\tansible_ssh_host=%s\tansible_ssh_user=ec2-user" % (
            args.app_name, i.id, i.private_ip_address))


def instance(args):
    ec2_conn = ec2.get_connection(args.profile, args.region)
    instances = ec2_conn.get_only_instances(filters={"tag:Name": args.name})

    for i in instances:
        if i.state != "running":
            continue

        i.ip_address = i.ip_address or ""

        print("%s %s %s" % (i.id,
                            i.ip_address.center(15),
                            i.private_ip_address.center(15)))


def run(argv):
    args = parameters.parse(parser=get_argument_parser(),
                            argv=argv,
                            postprocessors=[parameters.add_default_region])

    if args.env_name:
        env(args)
    else:
        instance(args)



from .. lib import (
    eb,
    parameters
)

import argparse


def get_argument_parser():
    parser = argparse.ArgumentParser("ebzl list")

    parameters.add_profile(parser)
    parameters.add_app_name(parser)
    parameters.add_region(parser, required=False)

    return parser


def list_versions(args):
    layer1 = eb.get_layer1(profile=args.profile,
                           region=args.region)
    data = layer1.describe_application_versions(application_name=args.app_name)

    versions = (data["DescribeApplicationVersionsResponse"]
                    ["DescribeApplicationVersionsResult"]
                    ["ApplicationVersions"])

    for version in versions:
        print(version["VersionLabel"])


def list_applications(args):
    layer1 = eb.get_layer1(profile=args.profile,
                           region=args.region)
    data = layer1.describe_applications()

    apps = (data['DescribeApplicationsResponse']
                ['DescribeApplicationsResult']
                ['Applications'])

    for app in apps:
        print(app["ApplicationName"])


def run(argv):
    args = parameters.parse(parser=get_argument_parser(),
                            argv=argv,
                            postprocessors=[parameters.add_default_region])

    if args.app_name:
        list_versions(args)
    else:
        list_applications(args)



import os
try:
    import configparser
except ImportError:
    import ConfigParser as configparser


class NoConfigFoundException(Exception):
    pass


def _get_config_parser(path):
    config_parser = configparser.ConfigParser()

    try:
        with open(os.path.expanduser(path), "rb") as f:
            config_parser.readfp(f)
    except IOError:
        return None
    else:
        return config_parser


def _get_credentials_data():
    """Return ebizzle's config."""

    config_paths = [
        "~/.aws/credentials"
    ]

    for config_path in config_paths:
        config_parser = _get_config_parser(config_path)
        if config_parser is not None:
            return config_parser
    raise NoConfigFoundException(
        "Can't find a config file in any of the locations: %s"
        % config_paths)


def _get_config_data(config_path="~/.aws/config"):
    parser = _get_config_parser(config_path)

    if parser is not None:
        return parser

    raise NoConfigFoundException("Can't find the config file: %s" 
                                 % config_path)


def get_credentials(profile):
    """Returns credentials for given profile as a (key, secret) tuple."""

    config = _get_credentials_data()

    key = config.get(profile, "aws_access_key_id")
    secret = config.get(profile, "aws_secret_access_key")

    return key, secret


def get_profile_names():
    """Get available profile names."""

    return _get_credentials_data().sections()


def get_default_region(profile):
    """Get the default region for given profile from AWS CLI tool's config."""

    config = _get_config_data()

    try:
        return config.get("profile %s" % profile, "region")
    except (configparser.NoSectionError, configparser.NoOptionError):
        pass

    try:
        return config.get("default", "region")
    except (configparser.NoSectionError, configparser.NoOptionError):
        pass

    return None



import os
import json
import zipfile
import argparse

from .. lib import (
    parameters
)


DEFAULT_SOURCE_BUNDLE_NAME = "source-bundle.zip"


def get_argument_parser():
    parser = argparse.ArgumentParser("ebzl bundle")

    parameters.add_profile(parser)
    parameters.add_app_name(parser)
    parameters.add_region(parser, required=False)

    parser.add_argument("-d", "--docker-image",
                        default=True,
                        help="Docker image name.")

    parser.add_argument("--s3-bucket",
                        default="build-deps",
                        help="AWS S3 bucket containing Docker Hub credentials.")

    parser.add_argument("--s3-key",
                        default="generic/docker/.dockercfg",
                        help="AWS S3 key name for Docker Hub credentials file.")

    parser.add_argument("-s", "--source-bundle",
                        default=".",
                        help="Source bundle destination path.")

    return parser


def get_docker_run_contents(args):
    data = {
        "AWSEBDockerrunVersion": "1",
        "Image": {
            "Name": "%s:%s" % (args.docker_image, args.version),
            "Update": "true"
        },
        "Authentication": {
            "Bucket": args.s3_bucket,
            "Key": args.s3_key
        },
        "Ports": [{
            "ContainerPort": "80"
        }],
        "Logging": "/home/dubizzle/logs/user"
    }

    return json.dumps(data, indent=4)


def get_source_bundle_file_path(args):
    path = os.path.abspath(os.path.expanduser(args.source_bundle))

    if os.path.isfile(path) and not args.force:
        raise ValueError("Already exists, pass --force to overwrite: %s" % path)
    
    elif os.path.isfile(path):
        return path
       
    elif os.path.isdir(path):
        return os.path.join(path, DEFAULT_SOURCE_BUNDLE_NAME)

    if not os.path.exists(path):
        dir_ = os.path.dirname(path)
        if not os.path.isdir(dir_):
            raise ValueError("%s does not exist." % dir_)
        return path

    raise NotImplementedError("I have no idea what I'm doing.")


def write_source_bundle(path, dockerrun_contents):
    with zipfile.ZipFile(path, "w") as f:
        f.writestr("Dockerrun.aws.json", dockerrun_contents)


def run(argv):
    args = parameters.parse(parser=get_argument_parser(),
                            argv=argv)

    try: 
        path = get_source_bundle_file_path(args)
    except ValueError as exc:
        print("Incorrect source bundle path: %s" % exc)
        exit()

    dockerrun_contents = get_docker_run_contents(args)
    write_source_bundle(path, dockerrun_contents)



from .. lib import (
    eb,
    parameters
)

import boto
import argparse


def get_argument_parser():
    parser = argparse.ArgumentParser("ebzl deploy")
    
    parameters.add_profile(parser)
    parameters.add_version_label(parser)
    parameters.add_region(parser, required=False)

    parser.add_argument("-e", "--env-name",
                        required=True,
                        help="ElasticBeanstalk environment name.")

    return parser


def deploy(args):
    kwargs = {
        "environment_name": args.env_name,
        "version_label": args.version
    }

    layer1 = eb.get_layer1(profile=args.profile, region=args.region)

    try:
        layer1.update_environment(**kwargs)
    except boto.exception.BotoServerError as exc:
        print(exc.message)


def run(argv):
    args = parameters.parse(parser=get_argument_parser(),
                            argv=argv,
                            postprocessors=[parameters.add_default_region])

    deploy(args)



from . import config

from boto import ec2


def get_region(region_name):
    for region in ec2.regions():
        if region.name == region_name:
            return region

    raise ValueError("Region not found: %s" % region_name)


def get_connection(profile, region):
    return ec2.connection.EC2Connection(*config.get_credentials(profile),
                                        region=get_region(region_name=region))



TEXT = "text"

BASH = "bash"

JSON = "json"

DOCKERENV = "dockerenv"

NAME_VALUE_DICT = "nvdict"

DEFAULT = TEXT

CHOICES = (TEXT, BASH, JSON, DOCKERENV, NAME_VALUE_DICT)


def print_dict(dictionary, format_=None):
    """Print a dictionary in a given format. Defaults to text."""

    format_ = format_ or DEFAULT

    if format_ == TEXT:
        for key, value in iter(sorted(dictionary.items())):
            print("%s = %s" % (key, value))
    elif format_ == DOCKERENV:
        for key, value in iter(sorted(dictionary.items())):
            print("%s=%s" % (key, value))
    elif format_ == BASH:
        for key, value in iter(sorted(dictionary.items())):
            print("export %s=%s" % (key, value))
    elif format_ == JSON:
        print(json.dumps(dictionary))
    elif format_ == NAME_VALUE_DICT:
        print("[")
        for key, value in iter(sorted(dictionary.items())):
            print('{"name": "%s", "value": "%s"},' % (key, value))
        print("]")


def print_list(list_, format_=None):
    """Print a list in a given format. Defaults to text."""

    format_ = format_ or DEFAULT

    if format_ == TEXT:
        for item in list_:
            print(item)
    elif format_ == JSON:
        print(json.dumps(list_))


def print_profile(profile, format_=None):
    """Print profile header."""

    format_ = format_ or DEFAULT

    if format_ == TEXT:
        print("[profile:%s]" % profile)
    elif format_ == BASH:
        print("# profile: %s" % profile)



#!/usr/bin/env python
from . import (
    config,
    format as fmt
)


APP_ARGS = ("-a", "--app-name")

APP_KWARGS = {
    "required": False,
    "help": "ElaticBeanstalk application name."
}


def add_profile(parser, required=True):
    parser.add_argument("-p", "--profile",
                        required=required,
                        choices=config.get_profile_names(),
                        help="AWS credentials profile.")


def add_app_name(parser, required=True):
    kwargs = APP_KWARGS
    kwargs["required"] = required

    parser.add_argument(*APP_ARGS, **kwargs)


def add_version_label(parser, required=True):
    parser.add_argument("-v", "--version",
                        required=required,
                        help="Version label.")


def add_region(parser, required=True):
    parser.add_argument("-r", "--region",
                        required=required,
                        help=("AWS region (only needed if the S3 buckets "
                              "needs to be created)."))


def add_force(parser):
    parser.add_argument("-F", "--force",
                        action="store_true",
                        help="Force, overwrite & burn things in the process.")


def add_format(parser, default=fmt.TEXT, choices=fmt.CHOICES):
    parser.add_argument("f", "--format",
                        default=default,
                        choices=choices,
                        help="Output format.")


def parse(parser, argv, postprocessors=None):
    postprocessors = postprocessors or []

    args = parser.parse_args(argv)

    for postprocessor in postprocessors:
        try:
            postprocessor(parser=parser, args=args)
        except Exception as exc:
            parser.print_usage()
            print(exc)
            exit()

    return args


def add_default_region(parser, args):
    if args.region:
        return

    args.region = config.get_default_region(args.profile)

    if args.region is None:
        raise Exception("Can't determine region based on AWS config file.")

    return args



import sys
import argparse

from .modules import (
    bundle,
    create,
    deploy,
    env,
    help as help_,
    instances,
    list as list_,
    version
)


_MODULES = {
    "bundle": bundle,
    "create": create,
    "deploy": deploy,
    "env": env,
    "help": help_,
    "instances": instances,
    "list": list_,
    "version": version
}


def get_module_list():
    return list(_MODULES.keys())


def get_module(name):
    return _MODULES.get(name)


def get_argument_parser():
    parser = argparse.ArgumentParser()

    parser.add_argument("action",
                        choices=get_module_list(),
                        help="Action to perform.")

    return parser


def main():
    parser = get_argument_parser()
    args = parser.parse_args(sys.argv[1:2])
    get_module(args.action).run(sys.argv[2:])


if __name__ == "__main__":
    main()



import argparse
import subprocess

def get_argument_parser():
    parser = argparse.ArgumentParser("ebzl version")

    return parser


def get_git_output(git_command):
    output = subprocess.check_output(git_command, shell=True)
    return filter(lambda x: x, output.split("\n"))[0]


def get_version():
    try:
        get_git_output("git log 2>/dev/null")
    except subprocess.CalledProcessError:
        raise ValueError("Not a git repository.")

    try:
        tag = get_git_output("git describe --exact-match --abbrev=0 2>/dev/null")
    except subprocess.CalledProcessError:
        tag = None

    short_hash = get_git_output(r"git log -1 --pretty=format:%h")

    if tag:
        return "%s-%s" % (tag, short_hash)

    return short_hash


def run(argv):
    try:
        print(get_version())
    except ValueError as exc:
        print("Error: %s" % exc)





import os
import argparse

from .. lib import (
    eb,
    s3,
    parameters
)

import boto


def get_argument_parser():
    parser = argparse.ArgumentParser("ebzl create")

    parameters.add_profile(parser)
    parameters.add_app_name(parser)
    parameters.add_version_label(parser)
    parameters.add_region(parser, required=False)
    parameters.add_force(parser)

    parser.add_argument("-s", "--source-bundle",
                        required=True,
                        help="ElasticBeanstalk source bundle archive.")

    parser.add_argument("--s3-bucket",
                        required=True,
                        help="Source bundle destination AWS S3 bucket.")

    return parser


def get_source_bundle_key_name(app_name, version):
    return "%s/%s.zip" % (app_name, version)


def upload_source_bundle(args):
    source_bundle_path = os.path.expanduser(args.source_bundle)

    s3_conn = s3.get_connection(profile_name=args.profile)

    try:
        bucket = s3_conn.get_bucket(args.s3_bucket)
    except boto.exception.S3ResponseError:
        print("S3 bucket not found, creating: %s" % args.s3_bucket)
        bucket = s3_conn.create_bucket(
            args.s3_bucket,
            location=s3.map_region_to_location(args.region))

    key = boto.s3.key.Key(
        bucket,
        get_source_bundle_key_name(args.app_name, args.version))

    if key.exists() and not args.force:
        print("Key already exists: %s (pass --force to overwrite)" % key.name)
        exit()

    with open(source_bundle_path, "rb") as f:
        key.set_contents_from_file(f)

    return key.key


def create_eb_version(args):
    layer1 = eb.get_layer1(profile=args.profile, region=args.region)

    kwargs = {
        "application_name": args.app_name,
        "version_label": args.version,
        "description": args.version,
        "s3_bucket": args.s3_bucket,
        "s3_key": get_source_bundle_key_name(args.app_name,
                                             args.version)
    }

    try:
        layer1.create_application_version(**kwargs)
    except boto.exception.BotoServerError as exc:
        print(exc.message)
        exit()


def run(argv):
    args = parameters.parse(parser=get_argument_parser(),
                            argv=argv,
                            postprocessors=[parameters.add_default_region])

    upload_source_bundle(args)
    create_eb_version(args)



from . import config

from boto import beanstalk
from boto.beanstalk import layer1


def get_region(region_name):
    for beanstalk_region in beanstalk.regions():
        if beanstalk_region.name == region_name:
            return beanstalk_region

    raise ValueError("Region not found: %s" % region_name)


def get_layer1(profile, region):
    return layer1.Layer1(*config.get_credentials(profile),
                         region=get_region(region))



from setuptools import setup, find_packages
from pip.req import parse_requirements
from pip.download import PipSession
from os import path


requirements = [str(ir.req) 
                for ir in parse_requirements('./requirements/requirements.txt', 
                                             session=PipSession())]

setup(
    name="ebzl",
    version="0.1",
    description="AWS ElasticBeanstalk management helper",
    url="http://github.com/bearops/ebzl",
    author='Wojciech Gaca',
    author_email='gaca@dubizzle.com',
    license="MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python"
    ],
    keywords=("aws amazon web services elastic beanstalk "
              "eb application environment"),
    packages=find_packages(),
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'ebzl = ebzl.ebzl:main'
        ]
    }
)



import argparse

from .. modules import (
    bundle,
    create,
    deploy,
    env,
    instances,
    list as list_,
    version
)


def get_module_list():
    return [
        "bundle",
        "create",
        "deploy",
        "env",
        "instances",
        "list",
        "version"
    ]


def get_module(name):
    d = {
        "bundle": bundle,
        "create": create,
        "deploy": deploy,
        "env": env,
        "instances": instances,
        "list": list_,
        "version": version
    }

    return d.get(name)


def get_argument_parser():
    parser = argparse.ArgumentParser("ebzl help")

    parser.add_argument("action",
                        help="Ebzl module.")

    return parser


def run(argv):
    args = get_argument_parser().parse_args(argv)
    try:
        get_module(args.action).get_argument_parser().print_help()
    except AttributeError:
        print("Module not found: %s" % args.action)
        exit()
